@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-## Copyright 2008-2012 Luc Saffre
+## Copyright 2008-2013 Luc Saffre
 ## This file is part of the Lino project.
 ## Lino is free software; you can redistribute it and/or modify 
 ## it under the terms of the GNU General Public License as published by
@@ -15,7 +15,8 @@
 """
 This module deserves more documentation.
 
-It defines tables like `Person` and `Company`
+It defines tables like 
+- :class:`Partner` (and their specializations :class:`Person` and :class:`Company`)
 
 """
 
@@ -46,11 +47,13 @@ from lino import dd
 from lino import mixins
 from lino.utils import join_words
 from lino.utils.choosers import chooser
-from lino.utils.choicelists import Gender
 from lino.utils import babel 
 #~ from lino.models import get_site_config
 
-from lino.modlib.countries.models import CountryCity
+#~ from lino.modlib.contacts.utils import Genders
+
+#~ from lino.modlib.countries.models import CountryCity
+from lino.modlib.countries.models import CountryRegionCity
 
 #~ from lino.modlib.contacts.utils import get_salutation
 #~ from lino.modlib.contacts.utils import GENDER_CHOICES, get_salutation
@@ -59,32 +62,6 @@ from lino.modlib.countries.models import CountryCity
 from lino.utils import mti
 
 
-def get_salutation(gender,nominative=False):
-    """
-    Returns "Mr" or "Mrs" or a translation thereof, 
-    depending on the gender and the current babel language.
-    
-    Note that the English abbreviations 
-    `Mr <http://en.wikipedia.org/wiki/Mr.>`_ and 
-    `Mrs <http://en.wikipedia.org/wiki/Mrs.>`_
-    are written either with (AE) or 
-    without (BE) a dot. Since the babel module doesn't yet allow 
-    to differentiate dialects, we opted for the british version.
-    
-    The optional keyword argument `nominative` used only when babel language
-    is "de": specifying ``nominative=True`` will return "Herr" instead of default 
-    "Herrn" for male persons.
-    
-    """
-    if not gender: return ''
-    if gender == Gender.female: return _("Mrs")
-    from django.utils.translation import pgettext
-    if nominative:
-        return pgettext("nominative salutation","Mr") 
-    return pgettext("indirect salutation","Mr") 
-    
-
-
 
 
 class CompanyType(babel.BabelNamed):
@@ -101,7 +78,7 @@ class CompanyType(babel.BabelNamed):
     
         
 class CompanyTypes(dd.Table):
-    required = dict(user_level='manager')
+    required = dd.required(user_level='manager')
     model = 'contacts.CompanyType'
     column_names = 'name *'
     #~ label = _("Company types")
@@ -110,8 +87,17 @@ class CompanyTypes(dd.Table):
 
 
 #~ class Contact(mti.MultiTableBase,CountryCity):
-class Partner(mti.MultiTableBase,CountryCity):
+class Partner(mti.MultiTableBase,CountryRegionCity):
     """
+    
+    A :class:`Partner` is anything that can act as a business partner.
+    A Partner has at least a name and usually also one "official" address.
+    Predefined subclasses of Partners are
+    :class:`Person` for physical persons and
+    :class:`Company` for companies, organisations and any kind of 
+    non-formal Partners.
+    
+    
     Base class for anything that has contact information 
     (postal address, email, phone,...).
     
@@ -120,11 +106,11 @@ class Partner(mti.MultiTableBase,CountryCity):
     """
     preferred width for ForeignKey fields to a Partner
     """
-    _lino_preferred_width = 20 
+    preferred_foreignkey_width = 20 
     
   
     class Meta:
-        #~ abstract = True
+        abstract = settings.LINO.is_abstract_model('contacts.Partner')
         verbose_name = _("Partner")
         verbose_name_plural = _("Partners")
   
@@ -149,10 +135,10 @@ class Partner(mti.MultiTableBase,CountryCity):
         max_length=200,blank=True,
         help_text="Address line to print below street line")
     
-    zip_code = models.CharField(_("Zip code"),
-        max_length=10,blank=True)
-    region = models.CharField(_("Region"),
-        max_length=200,blank=True)
+    #~ zip_code = models.CharField(_("Zip code"),
+        #~ max_length=10,blank=True)
+    #~ region = models.CharField(_("Region"),
+        #~ max_length=200,blank=True)
     language = babel.LanguageField()
     
     email = models.EmailField(_('E-Mail'),blank=True) # ,null=True)
@@ -163,6 +149,16 @@ class Partner(mti.MultiTableBase,CountryCity):
     
     remarks = models.TextField(_("Remarks"),blank=True) # ,null=True)
     
+    is_person = mti.EnableChild(
+        settings.LINO.person_model,
+        verbose_name=_("is Person"),
+        help_text=_("Whether this Partner is a Person."))
+        
+    is_company = mti.EnableChild(
+        settings.LINO.company_model,
+        verbose_name=_("is Company"),
+        help_text=_("Whether this Partner is a Company."))
+        
     def save(self,*args,**kw):
         if self.id is None:
             sc = settings.LINO.site_config # get_site_config()
@@ -186,6 +182,7 @@ Returns a one-line string representing this Partner.
 The default returns simply the `name` field, ignoring any parameters, 
 but e.g. :class:`PersonMixin` overrides this.
         """
+        #~ print '20120729 Partner.get_full_name`'
         
         #~ try:
             #~ p = getattr(self,'person')
@@ -268,21 +265,21 @@ class PartnerDetail(dd.FormLayout):
     bottom_box
     """
     
-    address_box = """
+    address_box = dd.Panel("""
     name_box
     country region city zip_code:10
     addr1
     street_prefix street:25 street_no street_box
     addr2
-    """
+    """,label = _("Address"))
     
-    contact_box = """
+    contact_box = dd.Panel("""
     info_box
     email:40 
     url
     phone
     gsm fax
-    """
+    """,label = _("Contact"))
 
     bottom_box = """
     remarks 
@@ -293,13 +290,14 @@ class PartnerDetail(dd.FormLayout):
     info_box = "id language"
     
     
-    def setup_handle(self,dh):
-        dh.address_box.label = _("Address")
-        dh.contact_box.label = _("Contact")
+    #~ def setup_handle(self,dh):
+        #~ dh.address_box.label = _("Address")
+        #~ dh.contact_box.label = _("Contact")
   
     
     
 class Partners(dd.Table):
+    required = dd.required(user_level='user')
     model = 'contacts.Partner'
     column_names = "name email * id" 
     order_by = ['name','id']
@@ -315,11 +313,11 @@ class Partners(dd.Table):
         return self.model.objects.select_related('country','city')
 
 
-class AllPartners(Partners):
+#~ class AllPartners(Partners):
   
-    @classmethod
-    def get_actor_label(self):
-        return _("All %s") % self.model._meta.verbose_name_plural
+    #~ @classmethod
+    #~ def get_actor_label(self):
+        #~ return _("All %s") % self.model._meta.verbose_name_plural
         
 class PartnersByCity(Partners):
     master_key = 'city'
@@ -333,118 +331,20 @@ class PartnersByCountry(Partners):
 
 
 
-class Born(dd.Model):
-    """
-    Abstract base class that adds a `birth_date` 
-    field and a virtual field "Age".
-    """
-    class Meta:
-        abstract = True
-        
-    birth_date = dd.IncompleteDateField(
-        blank=True,
-        verbose_name=_("Birth date"))
-        
-    #~ birth_date = models.DateField(
-        #~ blank=True,null=True,
-        #~ verbose_name=_("Birth date"))
-    #~ birth_date_circa = models.BooleanField(
-        #~ default=False,
-        #~ verbose_name=_("not exact"))
-    
-    def get_age_years(self,today=None):
-        if self.birth_date and self.birth_date.year:
-            if today is None:
-                today = datetime.date.today()
-            try:
-                return today - self.birth_date.as_date()
-            except ValueError:
-                pass
-      
-    @dd.displayfield(_("Age"))
-    def age(self,request,today=None):
-        a = self.get_age_years(today)
-        if a is None:
-            return unicode(_('unknown'))
-        s = _("%d years") % (a.days / 365)
-        if self.birth_date and self.birth_date.is_complete():
-            return s
-        return u"±" + s
-    
-    #~ @dd.displayfield(_("Age"))
-    #~ def age(self,request,today=None):
-        #~ if self.birth_date and self.birth_date.year:
-            #~ if today is None:
-                #~ today = datetime.date.today()
-            #~ try:
-                #~ dd = today - self.birth_date.as_date()
-            #~ except ValueError:
-                #~ pass
-            #~ else:
-                #~ s = _("%d years") % (dd.days / 365)
-                #~ if self.birth_date.is_complete():
-                    #~ return s
-                #~ return u"±" + s
-        #~ return _('unknown')
-    
-
-
-#~ Note `PersonMixin` must not be named `Person` because users.User also inherits 
-#~ from it and would then also find all contacs/Person/*.dtl !
-
-class PersonMixin(dd.Model):
+  
+class PersonMixin(mixins.Human):
     """
     Can be used also for Persons that are no Partners
     """
     class Meta:
         abstract = True
         
-    first_name = models.CharField(max_length=200,
-      #~ blank=True,
-      verbose_name=_('First name'))
-    "Space-separated list of all first names."
-    
-    last_name = models.CharField(max_length=200,
-      #~ blank=True,
-      verbose_name=_('Last name'))
-    """Last name (family name)."""
-    
     title = models.CharField(max_length=200,blank=True,
       verbose_name=_('Title'))
     """Text to print as part of the first address line in front of first_name."""
         
-    gender = Gender.field()
-        
-    def get_salutation(self,**salutation_options):
-        return get_salutation(
-            #~ translation.get_language(),
-            self.gender,**salutation_options)
-    
-        
-    def get_full_name(self,salutation=True,**salutation_options):
-        """Returns a one-line string composed of salutation, first_name and last_name.
-        
-The optional keyword argument `salutation` can be set to `False` 
-to suppress salutations. 
-See :func:`lino.apps.pcsw.tests.pcsw_tests.test04` 
-and
-:func:`lino.modlib.contacts.tests.test01` 
-for some examples.
-
-Optional `salutation_options` see :func:`get_salutation`.
-        """
-        #~ return '%s %s' % (self.first_name, self.last_name.upper())
-        words = []
-        if salutation:
-            words.append(self.get_salutation(**salutation_options))
-        words += [self.first_name, self.last_name.upper()]
-        return join_words(*words)
-    full_name = property(get_full_name)
-    #~ full_name.return_type = models.CharField(max_length=200,verbose_name=_('Full name'))
-    
   
-#~ class Person(dd.Model):
-class Person(Partner,PersonMixin):
+class Person(PersonMixin,Partner):
     """
     Mixin for models that represent a physical person. 
     """
@@ -463,15 +363,14 @@ class Person(Partner,PersonMixin):
         #~ yield  " ".join(l)
         
     def full_clean(self,*args,**kw):
+    #~ def save(self,*args,**kw):
         """
-        Set the :attr:`Partner.name` field of this person. 
+        Set the `name` field of this person. 
         This field is visible in the Partner's detail but not 
         in the Person's detail and serves for sorting 
         when selecting a Partner. 
         It also serves for quick search on Persons.
         """
-        #~ l = filter(lambda x:x,[self.last_name,self.first_name])
-        #~ self.name = " ".join(l)
         self.name = join_words(self.last_name,self.first_name)
         super(Person,self).full_clean(*args,**kw)
 
@@ -487,7 +386,8 @@ class PersonDetail(PartnerDetail):
     #~ """
     
     name_box = "last_name first_name:15 gender title:10"
-    info_box = "id:5 language:10 birth_date:10"
+    #~ info_box = "id:5 language:10 birth_date:10"
+    info_box = "id:5 language:10"
 
     bottom_box = "remarks contacts.RolesByPerson"
         
@@ -499,10 +399,12 @@ class PersonDetail(PartnerDetail):
   
 
 
-class Persons(dd.Table):
+#~ class Persons(dd.Table):
+class Persons(Partners):
     """
     List of all Persons.
     """
+    #~ required = dict(user_level='user')
     model = settings.LINO.person_model
     order_by = ["last_name","first_name","id"]
     column_names = "name_column:20 address_column email phone:10 gsm:10 id language:10 *"
@@ -541,6 +443,7 @@ class Company(Partner):
     #~ def get_full_name(self,**salutation_options):
     def get_full_name(self,salutation=True,**salutation_options):
         """Deserves more documentation."""
+        #~ print '20120729 Company.get_full_name`'
         if self.type:
             return join_words(self.type.abbr,self.name)
         return self.name
@@ -560,54 +463,68 @@ class CompanyDetail(PartnerDetail):
 
 
 
-              
 class Companies(Partners):
     model = settings.LINO.company_model
     order_by = ["name"]
     detail_layout = CompanyDetail()
     insert_layout = dd.FormLayout("""
-    name type
-    language email
-    """,window_size=(40,'auto'))
-    
-    
+    name 
+    language:20 email:40
+    type id
+    """,window_size=(60,'auto'))
     
+#~ class List(Partner):
+    #~ pass
+
+#~ class Lists(Partners):
+    #~ model = List
+    #~ order_by = ["name"]
+    #~ detail_layout = """
+    #~ id name
+    #~ language email
+    #~ MembersByList
+    #~ """
+    #~ insert_layout = dd.FormLayout("""
+    #~ name 
+    #~ language email
+    #~ """,window_size=(40,'auto'))
 
 
 
 # class ContactType(babel.BabelNamed):
 class RoleType(babel.BabelNamed):
     """
-    Deserves more documentation.
+    TODO: rename "RoleType" to "Function" or "ContactType".
+    
+    RoleType,name is used at "in seiner Eigenschaft als ..." 
+    in document templates for contracts.    
     """
     class Meta:
-        verbose_name = _("Role Type")
-        verbose_name_plural = _("Role Types")
+        verbose_name = _("Function")
+        verbose_name_plural = _("Functions")
 
 
 class RoleTypes(dd.Table):
-    required = dict(user_level='manager')
-    model = 'contacts.RoleType'
+    required = dd.required(user_level='manager')
+    model = RoleType
 
 
 #~ class Contact(dd.Model):
-#~ class RoleOccurence(dd.Model):
 class Role(dd.Model):
-#~ class unused_Role(object):
     """
-    The role of a given :class:`Person` in a given :class:`Company`.
+    
+    A Contact (historical model name :class:`Role`) 
+    is a :class:`Person` 
+    that has a given  role (:class:`ContactType`) 
+    in a given :class:`Company`. 
+    
+    TODO: rename "Role" to "Contact".
     """
   
     class Meta:
         verbose_name = _("Contact Person")
         verbose_name_plural = _("Contact Persons")
         
-    #~ parent = models.ForeignKey('contacts.Contact',
-        #~ verbose_name=_("Parent Contact"),
-        #~ related_name='rolesbyparent')
-    #~ child = models.ForeignKey('contacts.Contact',
-        #~ verbose_name=_("Child Contact"),
-        #~ related_name='rolesbychild')
     type = models.ForeignKey('contacts.RoleType',
       blank=True,null=True,
       verbose_name=_("Contact Role"))
@@ -664,19 +581,19 @@ class Role(dd.Model):
     #~ column_names = 'company type *'
     
 class Roles(dd.Table):
-    required = dict(user_level='manager')
+    required = dd.required(user_level='manager')
     #~ required_user_level = UserLevels.manager
     model = 'contacts.Role'   
     
 class RolesByCompany(Roles):
-    required = dict()
+    required = dd.required()
     #~ required_user_level = None
     label = _("Contact persons")
     master_key = 'company'
     column_names = 'person type *'
 
 class RolesByPerson(Roles):
-    required = dict()
+    required = dd.required()
     #~ required_user_level = None
     label = _("Contact for")
     master_key = 'person'
@@ -696,15 +613,10 @@ class PartnerDocument(dd.Model):
         abstract = True
         
     company = models.ForeignKey(settings.LINO.company_model,
-        blank=True,null=True,
-        #~ verbose_name=_("Company")
-        )
+        blank=True,null=True)
         
-    #~ person = models.ForeignKey("contacts.Person",
     person = models.ForeignKey(settings.LINO.person_model,
-        blank=True,null=True,
-        #~ verbose_name=_("Person")
-        )
+        blank=True,null=True)
         
     def get_partner(self):
         if self.company is not None:
@@ -768,6 +680,115 @@ class PartnerDocument(dd.Model):
         
 
 
+
+class OldCompanyContact(dd.Model):
+    """
+    Abstract class which adds two fields `company` and `contact`.
+    """
+    class Meta:
+        abstract = True
+        
+    company = models.ForeignKey(settings.LINO.company_model,
+        related_name="%(app_label)s_%(class)s_set_by_company",
+        verbose_name=_("Company"),
+        blank=True,null=True)
+        
+    contact = models.ForeignKey("contacts.Role",
+      related_name="%(app_label)s_%(class)s_set_by_contact",
+      blank=True,null=True,
+      verbose_name=_("represented by"))
+      
+    @chooser()
+    def contact_choices(cls,company):
+        if company is not None:
+            return cls.contact_choices_queryset(company)
+        return []
+        
+    @classmethod
+    def contact_choices_queryset(cls,company):
+        return Role.objects.filter(company=company)
+
+    def full_clean(self,*args,**kw):
+        if self.company:
+            if self.contact is None \
+              or self.contact.company is None \
+              or self.contact.company.pk != self.company.pk:
+                qs = self.contact_choices_queryset(self.company)
+                #~ qs = self.company.rolesbyparent.all()
+                if qs.count() == 1:
+                    self.contact = qs[0]
+                else:
+                    #~ print "20120227 clear contact!"
+                    self.contact = None
+        super(CompanyContact,self).full_clean(*args,**kw)
+
+
+class ContactRelated(dd.Model):
+    """
+    Abstract class for things that relate to a company represented by a person as a given role.
+    Adds 3 fields `company`, `contact_person` and `contact_role`.
+    """
+    class Meta:
+        abstract = True
+        
+    company = models.ForeignKey(settings.LINO.company_model,
+        related_name="%(app_label)s_%(class)s_set_by_company",
+        verbose_name=_("Company"),
+        blank=True,null=True)
+        
+    contact_person = models.ForeignKey("contacts.Person",
+      related_name="%(app_label)s_%(class)s_set_by_contact_person",
+      blank=True,null=True,
+      verbose_name=_("represented by"))
+      
+    contact_role = models.ForeignKey("contacts.RoleType",
+      related_name="%(app_label)s_%(class)s_set_by_contact_role",
+      blank=True,null=True,
+      verbose_name=_("represented as"))
+      
+    @chooser()
+    def contact_person_choices(cls,company):
+        if company is not None:
+            return cls.contact_person_choices_queryset(company)
+        return []
+        
+    def get_contact(self):
+        roles = Role.objects.filter(company=self.company,person=self.contact_person)
+        #~ print '20120929 get_contact', roles
+        if roles.count() == 1:
+            return roles[0]
+        
+    def contact_person_changed(self,ar):
+        #~ print '20120929 contact_person_changed'
+        if self.company and not self.contact_person_id:
+            roles = Role.objects.filter(company=self.company)
+            if roles.count() == 1:
+                self.contact_person = roles[0].person
+                self.contact_role = roles[0].type
+            return 
+      
+    @classmethod
+    def contact_person_choices_queryset(cls,company):
+    #~ def contact_choices_queryset(cls,company):
+        return settings.LINO.person_model.objects.filter(rolesbyperson__company=company).distinct()
+
+    def full_clean(self,*args,**kw):
+        if not settings.LINO.loading_from_dump:
+            if self.company and self.contact_person is None:
+                qs = self.contact_person_choices_queryset(self.company)
+                #~ qs = self.company.rolesbyparent.all()
+                if qs.count() == 1:
+                    self.contact_person = qs[0]
+                else:
+                    #~ print "20120227 clear contact!"
+                    self.contact = None
+            contact = self.get_contact()
+            if contact is not None:
+                self.contact_role = contact.type
+                #~ print '20120929b', contact.type
+        super(ContactRelated,self).full_clean(*args,**kw)
+
+
     
 
 
@@ -801,47 +822,57 @@ dd.inject_field(SiteConfig,
         blank=True,null=True,
         verbose_name=_("The company that runs this site"),
         related_name='site_company_sites',
-        help_text="""\
-The Company to be used as sender in documents."""
-        )
-    )
+        help_text=_("The Company to be used as sender in documents.")))
     
 
-dd.inject_field(Partner,
-    'is_person',
-    #~ mti.EnableChild('contacts.Person',verbose_name=_("is Person")),
-    mti.EnableChild(settings.LINO.person_model,verbose_name=_("is Person")),
-    """Whether this Partner is a Person."""
-    )
-dd.inject_field(Partner,
-    'is_company',
-    mti.EnableChild(settings.LINO.company_model,verbose_name=_("is Company")),
-    """Whether this Partner is a Company."""
-    )
+#~ dd.inject_field(Partner,
+    #~ 'is_person',
+    #~ mti.EnableChild(
+        #~ settings.LINO.person_model,
+        #~ verbose_name=_("is Person"),
+        #~ help_text=_("Whether this Partner is a Person.")))
+#~ dd.inject_field(Partner,
+    #~ 'is_company',
+    #~ mti.EnableChild(
+        #~ settings.LINO.company_model,
+        #~ verbose_name=_("is Company"),
+        #~ help_text=_("Whether this Partner is a Company.")))
 
 
-MODULE_NAME = _("Contacts")
+MODULE_LABEL = _("Contacts")
 
 def site_setup(site):
     site.modules.countries.Cities.set_detail_layout("""
     name country 
+    type parent zip_code id 
+    CitiesByCity
     contacts.PartnersByCity
     """)
+    
 
 
-def setup_main_menu(site,ui,user,m): pass
+def setup_main_menu(site,ui,profile,m):
+    m = m.add_menu("contacts",MODULE_LABEL)
+    #~ actors = (Persons,Companies,Partners)
+    #~ for m in (Person,Company,Partner):
+        #~ if m._meta.abstract: 
+            #~ return 
+    """
+    We use the string representations and not the classes because 
+    other installed applications may want to override these tables.
+    """
+    #~ for a in (Persons,Companies,Partners):
+    for a in ('contacts.Persons','contacts.Companies','contacts.Partners'):
+        m.add_action(a)
 
-def setup_my_menu(site,ui,user,m): 
+def setup_my_menu(site,ui,profile,m): 
     pass
   
-def setup_master_menu(site,ui,user,m): 
-    m = m.add_menu("contacts",MODULE_NAME)
-    m.add_action(Persons)
-    m.add_action(Companies)
-    m.add_action(Partners)
+def setup_master_menu(site,ui,profile,m): 
+    pass
     
-def setup_config_menu(site,ui,user,m): 
-    config_contacts = m.add_menu("contacts",MODULE_NAME)
+def setup_config_menu(site,ui,profile,m): 
+    config_contacts = m.add_menu("contacts",MODULE_LABEL)
     config_contacts.add_action(CompanyTypes)
     config_contacts.add_action(RoleTypes)
     config_contacts.add_action(site.modules.countries.Countries)
@@ -851,8 +882,8 @@ def setup_config_menu(site,ui,user,m):
     #~ m  = m.add_menu("contacts",_("~Contacts"))
     #~ m.add_action('contacts.RoleTypes')
   
-def setup_explorer_menu(site,ui,user,m):
-    m = m.add_menu("contacts",MODULE_NAME)
+def setup_explorer_menu(site,ui,profile,m):
+    m = m.add_menu("contacts",MODULE_LABEL)
     m.add_action(site.modules.contacts.Roles)
     m.add_action(site.modules.countries.Cities)
   
